"""Condition rule support for applications.

This module contains convenience imports for:

* :py:class:`~djblets.conditions.conditions.Condition`
* :py:class:`~djblets.conditions.conditions.ConditionSet`
"""

from djblets.conditions.conditions import Condition, ConditionSet


__all__ = [
    'Condition',
    'ConditionSet',
]
