a.copyable-text-input-link:hover,
a.copyable-text-input-link:link,
a.copyable-text-input-link:visited {
  color: #000;
  font-size: 1.5em;
  margin-left: 0.5em;
  vertical-align: middle;
}
/*# sourceMappingURL=copyable_text_input.css.map */