{% extends "base.html" %}
{% load djblets_deco pipeline %}

{% block title %}{{datagrid.title}}{% endblock %}

{% block extrahead %}
{%  stylesheet 'djblets-datagrid' %}
{%  include 'js/jquery.html' %}
{%  include 'js/jquery-ui.html' %}
{%  javascript 'djblets-gravy' %}
{%  javascript 'djblets-datagrid' %}
{% endblock %}

{% block content %}
{%  box %}
{{datagrid.render_listview}}
{%  endbox %}
{% endblock %}
